/**
 * Copyright (c) 2007, 2020 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *    Dmitry Stadnik (Borland) - initial API and implementation
 * 	  Michael Golubev (Montages) - API extracted to GMF-T runtime, migrated to Xtend2
 *    Aurelien Didier (ARTAL) - aurelien.didier51@gmail.com - Bug 569174
 *    Etienne Allogo (ARTAL) - etienne.allogo@artal.fr - Bug 569174 : L1.2 clean up
 */
package xpt.expressions;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.util.Objects;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenExpressionInterpreter;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenLanguage;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import xpt.Common;

@Singleton
@SuppressWarnings("all")
public class RegexpExpressionFactory {
  @Inject
  @Extension
  private Common _common;

  @Inject
  private ExpressionAbstractExpression xptAbstractExpression;

  @Inject
  private OCLExpressionFactory xptOCLExpressionFactory;

  public CharSequence className(final GenExpressionInterpreter it) {
    StringConcatenation _builder = new StringConcatenation();
    String _className = it.getClassName();
    _builder.append(_className);
    return _builder;
  }

  public CharSequence packageName(final GenExpressionInterpreter it) {
    StringConcatenation _builder = new StringConcatenation();
    String _expressionsPackageName = it.getContainer().getExpressionsPackageName();
    _builder.append(_expressionsPackageName);
    return _builder;
  }

  public CharSequence qualifiedClassName(final GenExpressionInterpreter it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(".");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    return _builder;
  }

  public CharSequence fullPath(final GenExpressionInterpreter it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _qualifiedClassName = this.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    return _builder;
  }

  public CharSequence RegexpExpressionFactory(final GenExpressionInterpreter it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _copyright = this._common.copyright(it.getContainer().getEditorGen());
    _builder.append(_copyright);
    _builder.newLineIfNotEmpty();
    _builder.append("package ");
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _generatedClassComment = this._common.generatedClassComment();
    _builder.append(_generatedClassComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _initInterpreterFactory = this.xptOCLExpressionFactory.initInterpreterFactory(it);
    _builder.append(_initInterpreterFactory, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public static ");
    CharSequence _qualifiedClassName = this.xptAbstractExpression.qualifiedClassName(it.getContainer().getEditorGen().getDiagram());
    _builder.append(_qualifiedClassName, "\t");
    _builder.append(" getExpression(String body, org.eclipse.emf.ecore.EClassifier context, java.util.Map<String, org.eclipse.emf.ecore.EClassifier> environment) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return new Expression(body, context, environment);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment("This method will become private in the next release");
    _builder.append(_generatedMemberComment_1, "\t");
    _builder.append("/* FIXME [MG] private or completely remove in the next release  */");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public static ");
    CharSequence _qualifiedClassName_1 = this.xptAbstractExpression.qualifiedClassName(it.getContainer().getEditorGen().getDiagram());
    _builder.append(_qualifiedClassName_1, "\t");
    _builder.append(" getExpression(String body, org.eclipse.emf.ecore.EClassifier context) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return getExpression(body, context, java.util.Collections.<String, org.eclipse.emf.ecore.EClassifier>emptyMap());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_2 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_2, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private static class Expression extends ");
    CharSequence _qualifiedClassName_2 = this.xptAbstractExpression.qualifiedClassName(it.getContainer().getEditorGen().getDiagram());
    _builder.append(_qualifiedClassName_2, "\t");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_3 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_3, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("private final java.util.regex.Pattern pattern;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_4 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_4, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("@SuppressWarnings(\"rawtypes\")");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("public Expression(String body, org.eclipse.emf.ecore.EClassifier context, java.util.Map environment) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("super(body, context);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("java.util.regex.Pattern p;");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("try {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("p = java.util.regex.Pattern.compile(body);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("} catch (java.util.regex.PatternSyntaxException e) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("setStatus(org.eclipse.core.runtime.IStatus.ERROR, e.getMessage(), e);");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("p = null;");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("this.pattern = p;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_5 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_5, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("@SuppressWarnings(\"rawtypes\")");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("protected Object doEvaluate(Object contextInstance, java.util.Map env) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("if (pattern == null) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("if (context() instanceof org.eclipse.emf.ecore.EDataType) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("contextInstance = org.eclipse.emf.ecore.util.EcoreUtil.convertToString((org.eclipse.emf.ecore.EDataType) context(), contextInstance);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("java.util.regex.Matcher matcher = this.pattern.matcher(String.valueOf(contextInstance));");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return Boolean.valueOf(");
    {
      GenLanguage _language = it.getLanguage();
      boolean _equals = Objects.equals(_language, GenLanguage.NREGEXP_LITERAL);
      if (_equals) {
        _builder.append("!");
      }
    }
    _builder.append("matcher.matches());");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
